#!/bin/sh
log=/var/log/nicfix.log

function fix_eeprom()
{
   eth=$1
   if [ -z "$eth" ]; then
      return 1;
   fi
   if ! ifconfig $eth > /dev/null; then
      echo "$eth does not exist" 
      return 1;
   fi
   bdf=$(ethtool -i $eth | grep "bus-info:" | awk '{print $2}')
   dev=$(lspci -s $bdf -x | grep "00: 86 80" | awk '{print "0x"$5$4$3$2}')
   case $dev in
	0x10d38086)
		echo "$eth: is a 82574L Gigabit Network Connection"
		;;
	0x10f68086)
		echo "$eth: is a 82574L Gigabit Network Connection" 
		;;   
	0x150c8086)
		echo "$eth: is a 82583V Gigabit Network Connection" 
		;;
	*)
		echo "No appropriate hardware found for this fixup" 
		return 2
		;;
   esac
   echo "This fixup is applicable to your hardware" 
   var=$(ethtool -e $eth | grep 0x0010 | awk '{print $16}')
   new=$(echo ${var:0:1}`echo ${var:1} | tr '014589bc' '2367abef'`)
   if [ ${var:0:1}${var:1} == $new ]; then
	echo "Your eeprom is up to date, no changes were made" 
	return 0
   fi
   echo "Applying Intel NIC EEPROM fix ..."
   echo "executing: ethtool -E $eth magic $dev offset 0x1e value 0x$new" 
   ethtool -E $1 magic $dev offset 0x1e value 0x$new
   echo "Change made. You must reboot before changes take effect." 
   return 0
}

function ethmatch()
{
   bdf=$(ethtool -i $1 | grep "bus-info:" | awk '{print $2}')
   dev=$(lspci -s $bdf -x | grep "00: 86 80" | awk '{print "0x"$5$4$3$2}')
   case $dev in
        0x10d38086)
                echo "$1: is a 82574L Gigabit Network Connection"
                ret=1
                ;;
        0x10f68086)
                echo "$1: is a 82574L Gigabit Network Connection"
                ret=1
                ;;
        0x150c8086)
                echo "$1: is a 82583V Gigabit Network Connection" 
                ret=1
                ;;
        *)
                echo "$1: is not a match" 
                ret=0
                ;;
   esac
   return $ret
}

lspci |grep Eth |grep -q 82574
i1=$?
lspci |grep Eth |grep -q 82583
i2=$?
if [ $i1 -eq 0 -o $i2 -eq 0 ]; then
   # found matching NICs, set kernel param
   grubby --update-kernel="$( grubby --default-kernel )" --args="pcie_aspm=off"

   # find which eth device matches these NICs
   doeth=eth0
   ethlist=`ifconfig |grep eth |awk '{ print $1 }' |sed -e 's/\n/ /g'`
   echo "$0 started" >$log
   for eth in $ethlist
   do
     echo "check $eth ..."  >>$log
     ethmatch $eth   >>$log
     if  [ $? -eq 1 ]; then
        echo "ethmatch $eth found"  >>$log
	doeth=$eth
	break
     fi
   done
   echo "matching eth = $doeth" >>$log

   # do the EEPROM fixup from Intel
   fix_eeprom $doeth  >>$log
fi
